@@ -105,11 +105,14 @@ struct _HDApplicationsMenuPrivate
   GnomeVFSMonitorHandle    *home_dir_monitor;
   GnomeVFSMonitorHandle    *desktop_dir_monitor;
   guint                     monitor_update_timeout;
+  guint                     menu_update_timeout;
   gboolean                  focus_applications;
   gboolean                  motion_reset;
 };
 
 static void hd_applications_menu_register_monitors (HDApplicationsMenu *button);
+static void hd_applications_menu_initial_setup (HDApplicationsMenu *button);
+static gboolean hd_applications_menu_changed (HDApplicationsMenu *button);
 static void hd_applications_menu_create_menu (HDApplicationsMenu *button);
 static void hd_applications_menu_button_toggled (GtkWidget *widget, HDApplicationsMenu *button);
 static gboolean hd_applications_menu_button_button_press (GtkWidget *widget, GdkEventButton *event, HDApplicationsMenu *button);
@@ -213,6 +216,9 @@ hd_applications_menu_init (HDApplicationsMenu *button)
   gtk_container_add (GTK_CONTAINER (button), priv->button);
 
   hd_applications_menu_register_monitors (button);
+
+  priv->menu_update_timeout =
+      g_timeout_add (5500, (GSourceFunc) hd_applications_menu_changed, button);
 }
 
 GtkWidget *
@@ -250,6 +256,8 @@ hd_applications_menu_popdown (GtkWidget *menu, HDApplicationsMenu *button)
 
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button->priv->button),
 			        FALSE);
+
+  hd_applications_menu_initial_setup (button);
 }
 
 static gboolean
@@ -360,7 +368,7 @@ hd_applications_menu_key_press (GtkWidget *menu,
       return gtk_widget_event (GTK_WIDGET (button->priv->menu_applications), (GdkEvent *) event);
     }
   }
-  
+ 
   return FALSE;
 }
 
@@ -370,14 +378,17 @@ hd_applications_menu_activate_category (GtkMenuItem *item, HDApplicationsMenu *b
   static GtkMenuItem *last_selected_item = NULL;
 
   GList *sub_items, *i;
-
+  gboolean replaced = FALSE;
+  
   sub_items = hildon_desktop_popup_menu_get_children 
 	  (button->priv->menu_applications);
   
   if (last_selected_item != item)
   {
     last_selected_item = item;
-	  
+
+    replaced = TRUE;
+    
     for (i = g_list_first (sub_items); i; i = i->next)
     {
       GtkMenuItem *child = (GtkMenuItem *) i->data;
@@ -388,7 +399,7 @@ hd_applications_menu_activate_category (GtkMenuItem *item, HDApplicationsMenu *b
     g_list_free (sub_items);
 
     sub_items = (GList *) g_object_get_data (G_OBJECT (item),
-          	                             CATEGORY_SUB_ITEMS);
+                                             CATEGORY_SUB_ITEMS);
 
     hildon_desktop_popup_menu_replace_items (button->priv->menu_applications, 
 		    		  	     sub_items);
@@ -411,6 +422,9 @@ hd_applications_menu_activate_category (GtkMenuItem *item, HDApplicationsMenu *b
   }
 
   button->priv->focus_applications = FALSE;
+
+  if (!replaced)
+    g_list_free (sub_items);
 }
 
 static void
@@ -632,9 +646,18 @@ hd_applications_menu_get_items (HDApplicationsMenu *button,
       else
       {
         /* Add "no applications" item */
-        GtkWidget *no_app_item;
+        GtkWidget *no_app_item, *label;
 
-        no_app_item = gtk_image_menu_item_new_with_label (MENU_ITEM_EMPTY_SUBMENU_STRING);
+        label = gtk_label_new (MENU_ITEM_EMPTY_SUBMENU_STRING);
+     
+        gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
+        gtk_misc_set_alignment (GTK_MISC (label), 0.5, 0.5);
+
+        gtk_widget_show (label);
+	
+        no_app_item = gtk_menu_item_new ();
+
+        gtk_container_add (GTK_CONTAINER (no_app_item), label);
 
         gtk_widget_set_sensitive (no_app_item, FALSE);
 	
@@ -842,15 +865,21 @@ hd_applications_menu_create_menu (HDApplicationsMenu *button)
 {
   HildonDesktopPopupWindow *popup_window;
   const GtkWidget *button_up, *button_down;
-  GtkWidget *box, *alignment, *arrow;
+  GtkWidget *event_box, *box, *alignment, *arrow;
   
   g_return_if_fail (button);
 
+  if (button->priv->menu_update_timeout > 0)
+  {
+    g_source_remove (button->priv->menu_update_timeout);
+    button->priv->menu_update_timeout = 0;
+  }
+
   g_signal_handlers_block_by_func (button->priv->button,
   				   hd_applications_menu_button_toggled, 
   				   button);
 
-  box = gtk_hbox_new (TRUE, 0);
+  box = gtk_hbox_new (FALSE, 0);
   
   popup_window =
     HILDON_DESKTOP_POPUP_WINDOW 
@@ -864,7 +893,7 @@ hd_applications_menu_create_menu (HDApplicationsMenu *button)
   
   gtk_widget_set_name (GTK_WIDGET (popup_window), HD_APPS_MENU_POPUP_WINDOW_NAME);
 
-  gtk_widget_set_size_request (GTK_WIDGET (popup_window), 640, 445);
+  gtk_widget_set_size_request (GTK_WIDGET (popup_window), 665, 445);
 
   /* We don't attach the widget because if we do it, we cannot be on top of 
    * virtual keyboard. Anyway it should be transient to button->priv->button
@@ -873,6 +902,12 @@ hd_applications_menu_create_menu (HDApplicationsMenu *button)
   hildon_desktop_popup_window_attach_widget (popup_window, NULL);
 #endif
 
+  event_box = gtk_event_box_new ();
+
+  gtk_widget_set_name (event_box, HD_APPS_MENU_CAT_MENU_NAME);
+
+  gtk_widget_set_size_request (GTK_WIDGET (event_box), 290, -1);
+
   button->priv->menu_categories =
     HILDON_DESKTOP_POPUP_MENU (g_object_new (HILDON_DESKTOP_TYPE_POPUP_MENU,
 		  		    	     "item-height", 72,
@@ -890,9 +925,14 @@ hd_applications_menu_create_menu (HDApplicationsMenu *button)
   gtk_widget_set_name (GTK_WIDGET (button->priv->menu_categories), 
   		       HD_APPS_MENU_CAT_MENU_NAME);
 
+  gtk_widget_show (event_box);
+
+  gtk_container_add (GTK_CONTAINER (event_box), 
+		     GTK_WIDGET (button->priv->menu_categories));
+
   gtk_box_pack_start (GTK_BOX (box),
-		      GTK_WIDGET (button->priv->menu_categories),
-		      TRUE,
+		      event_box,
+		      FALSE,
 		      TRUE,
 		      0); 
   
@@ -914,6 +954,10 @@ hd_applications_menu_create_menu (HDApplicationsMenu *button)
   gtk_container_add (GTK_CONTAINER (button_down), arrow);
   gtk_widget_show (arrow);
 
+  event_box = gtk_event_box_new ();
+
+  gtk_widget_set_name (event_box, HD_APPS_MENU_APP_MENU_NAME);
+
   button->priv->menu_applications =
     HILDON_DESKTOP_POPUP_MENU (g_object_new (HILDON_DESKTOP_TYPE_POPUP_MENU,
 		  		    	     "item-height", 72,
@@ -926,8 +970,13 @@ hd_applications_menu_create_menu (HDApplicationsMenu *button)
   gtk_widget_set_name (GTK_WIDGET (button->priv->menu_applications), 
   		       HD_APPS_MENU_APP_MENU_NAME);
 
+  gtk_widget_show (event_box);
+
+  gtk_container_add (GTK_CONTAINER (event_box), 
+		     GTK_WIDGET (button->priv->menu_applications));
+  
   gtk_box_pack_start (GTK_BOX (box),
-		      GTK_WIDGET (button->priv->menu_applications),
+		      event_box,
 		      TRUE,
 		      TRUE,
 		      0); 
@@ -1104,21 +1153,13 @@ hd_applications_menu_get_menu_position (GtkMenu *menu,
 }
 
 static void
-hd_applications_menu_show (HDApplicationsMenu * button)
+hd_applications_menu_initial_setup (HDApplicationsMenu *button)
 {
   GList *menu_items;
   
   g_return_if_fail (button);
   
-  if (!button->priv->popup_window)
-    hd_applications_menu_create_menu (button);
-
   button->priv->motion_reset = TRUE;
-  
-  hildon_desktop_popup_window_popup (button->priv->popup_window,
-		                     (HDPopupWindowPositionFunc) hd_applications_menu_get_menu_position,
-		                     button,
-		                     GDK_CURRENT_TIME);
 
   hildon_desktop_popup_menu_select_first_item (button->priv->menu_categories);
   hildon_desktop_popup_menu_scroll_to_selected (button->priv->menu_categories);
@@ -1133,6 +1174,23 @@ hd_applications_menu_show (HDApplicationsMenu * button)
 
   g_list_free (menu_items);
 }
+	
+static void
+hd_applications_menu_show (HDApplicationsMenu *button)
+{
+  g_return_if_fail (button);
+  
+  if (!button->priv->popup_window)
+  {
+    hd_applications_menu_create_menu (button);
+    hd_applications_menu_initial_setup (button);
+  }
+    
+  hildon_desktop_popup_window_popup (button->priv->popup_window,
+		                     (HDPopupWindowPositionFunc) hd_applications_menu_get_menu_position,
+		                     button,
+		                     GDK_CURRENT_TIME);
+}
 
 static void
 hd_applications_menu_button_toggled (GtkWidget *widget, HDApplicationsMenu *button)
@@ -1220,20 +1278,22 @@ hd_applications_menu_changed (HDApplicationsMenu *button)
   
   hd_applications_menu_create_menu (button);
   
+  hd_applications_menu_initial_setup (button);
+
   return FALSE;
 }
 
 static void
 hd_applications_menu_dir_changed (GnomeVFSMonitorHandle *handle,
-                              const gchar *monitor_uri,
-                              const gchar *info_uri,
-                              GnomeVFSMonitorEventType event_type,
-                              HDApplicationsMenu *button)
+                                  const gchar *monitor_uri,
+                                  const gchar *info_uri,
+                                  GnomeVFSMonitorEventType event_type,
+                                  HDApplicationsMenu *button)
 {
   if (!button->priv->monitor_update_timeout)
   {
     button->priv->monitor_update_timeout =
-            g_timeout_add (1000,
+            g_timeout_add (500,
             	           (GSourceFunc) hd_applications_menu_changed,
             	           button);
   }
